<?php
/**
 * Anwiki is a multilingual content management system <http://www.anwiki.com>
 * Copyright (C) 2007-2008 Antoine Walter <http://www.anw.fr>
 * 
 * Anwiki is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 * 
 * Anwiki is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Anwiki.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * ContentField definition for components settings.
 * @package Anwiki
 * @version $Id: class_contentfield_settings.php,v 1.2 2008-09-11 22:42:02 antoine-cvs Exp $
 * @copyright 2007-2008 Antoine Walter
 * @license http://www.gnu.org/copyleft/gpl.html GNU Public License 3
 */

abstract class AnwContentFieldSettings extends AnwStructuredContentField
{
	function __construct($sName)
	{
		parent::__construct($sName);
	}
	
	function t_structuredcontentfield($id, $asParams=array(), $sLangIfLocal=false, $sValueIfNotFound=false)
	{
		if ($this->getRelatedComponent())
		{
			$oComponentForTranslation = $this->getRelatedComponent();
		}
		else
		{
			$oComponentForTranslation = $this->getComponent();
		}
		return $oComponentForTranslation->t_contentfieldsettings($id, $asParams, $sLangIfLocal, $sValueIfNotFound);
	}
	
	function getComponent()
	{
		return $this->getContentClass()->getComponent();
	}
	
	protected function getFieldTranslation($sTranslationName, $asParameters=array(), $sTranslationIfEmpty=false)
	{
		//first, try to get it from component under "setting_"...
		$sTranslation = $this->doGetFieldTranslation("setting", $sTranslationName, $asParameters, true);
		
		//if not found, run parent
		if ($sTranslation === "")
		{
			$sTranslation = parent::getFieldTranslation($sTranslationName, $asParameters, $sTranslationIfEmpty);
		}
		return $sTranslation;
	}
	
	//
	// BELOW : Stuff artificially INHERITED from AnwStructuredContentFieldsContainerSettings
	// (multiple inherit is not supported by PHP5)
	//
	
	final function rebuildContentFromXml($sDefaultValue)
	{
		return AnwContentSettings::rebuildContentFromXml($this, $sDefaultValue);
	}
	
	final protected function testContentFieldMultiplicity($asFieldValues)
	{
		if (count($asFieldValues)==0 && !$this->isMultiple() && !$this->isMandatory())
		{
			//ignore errors, it means that default settings were not overloaded
		}
		else
		{
			$this->getMultiplicity()->testContentFieldMultiplicity($this, $asFieldValues);
		}
	}
	
	//
	//End of Stuff artificially INHERITED from AnwStructuredContentFieldsContainerSettings
	//
	
	
	final function testContentFieldValue($mContentFieldValue)
	{
		//tests related to contentfield type
		$this->doTestContentFieldValue($mContentFieldValue);
	}
	
	//overridable
	protected function doTestContentFieldValue($mContentFieldValue){}	
	
	//overridable
	function getValueFromConfigItem($mCfgItem)
	{
		return $mCfgItem;
	}
	function getConfigItemFromValue($mValue)
	{
		return $mValue;
	}
}

/**
 * Grouping some code for contentfield based on a Datatype.
 */
abstract class AnwContentFieldSettings_datatype extends AnwContentFieldSettings implements AnwStructuredContentField_atomic
{
	private $oData=null;
	
	//needs to be overriden
	abstract function initDatatype();
	
	protected function getDatatype()
	{
		if ($this->oData === null)
		{
			$this->oData = $this->initDatatype();
		}
		return $this->oData;
	}
	
	protected function renderEditInput($sInputValue, $sSuffix, $nInputNumber)
	{
		$sInputName = $this->getInputName($sSuffix);
		$sInputId = $this->getInputId($sInputName, $nInputNumber);
		
		$HTML = $this->getDatatype()->renderInput($sInputName, $sInputValue, $sInputId, array());
		return $HTML;
	}
	
	protected function renderCollapsedInput($sInputValue, $sSuffix)
	{
		$sInputName = $this->getInputName($sSuffix);		
		$HTML = $this->getDatatype()->renderCollapsedInput($sInputName, $sInputValue, array());
		return $HTML;
	}
	
	function doTestContentFieldValue($mContentFieldValue)
	{
		$this->getDatatype()->testValue($mContentFieldValue);
	}
	
	function getValuesFromPost($sSuffix)
	{
		$sInputName = $this->getInputName($sSuffix);
		$asValues = $this->getDatatype()->getValuesFromPost($sInputName);
		return $asValues;
	}
	
	function getFieldTip($asParameters=array())
	{
		return $this->getDatatype()->getTip($asParameters=array());
	}
	
	function setDefaultValue($sValue)
	{
		$this->getDatatype()->setDefaultValue($sValue);
	}
	
	function getDefaultValue()
	{
		return $this->getDatatype()->getDefaultValue();
	}
	
	function hasDefaultValue()
	{
		return $this->getDatatype()->hasDefaultValue();
	}
	
	function getValueFromConfigItem($mCfgItem)
	{
		return $this->getDatatype()->getValueFromConfigItem($mCfgItem);
	}
	function getConfigItemFromValue($mValue)
	{
		return $this->getDatatype()->getConfigItemFromValue($mValue);
	}
}

class AnwContentFieldSettings_xml extends AnwContentFieldSettings_datatype
{
	function initDatatype()
	{
		$oDatatype = new AnwDatatype_xml();
		$oDatatype->setDefaultValue("");
		return $oDatatype;
	}
	
	function setCheckPhpSyntax($bValue)
	{
		$this->getDatatype()->setCheckPhpSyntax($bValue);
	}
	
	protected function renderEditInput($sInputValue, $sSuffix, $nInputNumber)
	{
		$sInputName = $this->getInputName($sSuffix);
		$sInputId = $this->getInputId($sInputName, $nInputNumber);
		
		$asInputParameters = array(
			'onkeypress' => "textareaKeyPress(event)",
			'dir' => AnwComponent::g_("local_html_dir")
		);
		
		$HTML = $this->getDatatype()->renderInput($sInputName, $sInputValue, $sInputId, $asInputParameters);
		return $HTML;
	}
}

class AnwContentFieldSettings_xhtml extends AnwContentFieldSettings_xml
{
	function initDatatype()
	{
		$oDatatype = new AnwDatatype_xhtml();
		$oDatatype->setDefaultValue("");
		return $oDatatype;
	}
}

class AnwContentFieldSettings_date extends AnwContentFieldSettings_datatype
{
	function initDatatype()
	{
		$oDatatype = new AnwDatatype_date();
		$oDatatype->setDefaultValue( AnwDatatype_date::formatTime(time()) );
		return $oDatatype;
	}
	
	protected function renderEditInput($sInputValue, $sSuffix, $nInputNumber)
	{
		$sInputName = $this->getInputName($sSuffix);
		$sInputId = $this->getInputId($sInputName, $nInputNumber);
		
		$asInputParameters = array(
			'dir' => AnwComponent::g_("local_html_dir")
		);
		
		$HTML = $this->getDatatype()->renderInput($sInputName, $sInputValue, $sInputId, $asInputParameters);
		return $HTML;
	}
}

class AnwContentFieldSettings_string extends AnwContentFieldSettings_datatype
{
	function initDatatype()
	{
		$oDatatype = new AnwDatatype_string();
		$oDatatype->setDefaultValue("");
		return $oDatatype;
	}
	
	protected function renderEditInput($sInputValue, $sSuffix, $nInputNumber)
	{
		$sInputName = $this->getInputName($sSuffix);
		$sInputId = $this->getInputId($sInputName, $nInputNumber);
		
		$asInputParameters = array(
			'dir' => AnwComponent::g_("local_html_dir")
		);
		
		$HTML = $this->getDatatype()->renderInput($sInputName, $sInputValue, $sInputId, $asInputParameters);
		return $HTML;
	}
	
	function addAllowedPattern($sPattern)
	{
		$this->getDatatype()->addAllowedPattern($sPattern);
	}
}

class AnwContentFieldSettings_password extends AnwContentFieldSettings_string
{
	function initDatatype()
	{
		$oDatatype = new AnwDatatype_password();
		$oDatatype->setDefaultValue("");
		return $oDatatype;
	}
}

class AnwContentFieldSettings_ip_address extends AnwContentFieldSettings_string
{
	function initDatatype()
	{
		$oDatatype = new AnwDatatype_ip_address();
		$oDatatype->setDefaultValue("");
		return $oDatatype;
	}
}

class AnwContentFieldSettings_email_address extends AnwContentFieldSettings_string
{
	function initDatatype()
	{
		$oDatatype = new AnwDatatype_email_address();
		$oDatatype->setDefaultValue("");
		return $oDatatype;
	}
}

class AnwContentFieldSettings_url extends AnwContentFieldSettings_string
{
	private $amAllowedUrlTypes;
	
	function __construct($sName, $amAllowedUrlTypes=false)
	{
		$this->amAllowedUrlTypes = $amAllowedUrlTypes;
		parent::__construct($sName);
	}
	
	function initDatatype()
	{
		$oDatatype = new AnwDatatype_url();
		$oDatatype->setDefaultValue("");
		if ($this->amAllowedUrlTypes)
		{
			$oDatatype->setAllowedUrlTypes($this->amAllowedUrlTypes);
		}
		return $oDatatype;
	}
}

class AnwContentFieldSettings_system_directory extends AnwContentFieldSettings_string
{
	function initDatatype()
	{
		$oDatatype = new AnwDatatype_system_directory();
		$oDatatype->setDefaultValue("");
		return $oDatatype;
	}
	
	function addTestFile($sTestFile)
	{
		$this->getDatatype()->addTestFile($sTestFile);
	}
}

class AnwContentFieldSettings_userlogin extends AnwContentFieldSettings_string
{
	function initDatatype()
	{
		$oDatatype = new AnwDatatype_userlogin();
		$oDatatype->setDefaultValue("");
		return $oDatatype;
	}
}

class AnwContentFieldSettings_boolean extends AnwContentFieldSettings_datatype
{
	function initDatatype()
	{
		$oDatatype = new AnwDatatype_boolean();
		$oDatatype->setDefaultValue( AnwDatatype_boolean::formatBoolean(false) );
		return $oDatatype;
	}
	
	function init()
	{
		$this->getDatatype()->setLabel( $this->getFieldTranslation("_checkbox") );
	}
}

abstract class AnwContentFieldSettings_number extends AnwContentFieldSettings_datatype
{
	function setValueMin($nValue)
	{
		$this->getDatatype()->setValueMin($nValue);
	}
	
	function setValueMax($nValue)
	{
		$this->getDatatype()->setValueMax($nValue);
	}
}

class AnwContentFieldSettings_integer extends AnwContentFieldSettings_number
{
	function initDatatype()
	{
		$oDatatype = new AnwDatatype_integer();
		$oDatatype->setDefaultValue(0);
		return $oDatatype;
	}
	
	function init()
	{
		$this->getDatatype()->setLabel( $this->getFieldTranslation("_number",array(),"") );
	}
}

class AnwContentFieldSettings_delay extends AnwContentFieldSettings_integer
{
	function initDatatype()
	{
		$oDatatype = new AnwDatatype_delay();
		$oDatatype->setDefaultValue(0);
		return $oDatatype;
	}
}

class AnwContentFieldSettings_float extends AnwContentFieldSettings_number
{
	function initDatatype()
	{
		$oDatatype = new AnwDatatype_float();
		$oDatatype->setDefaultValue(0.0);
		return $oDatatype;
	}
	
	function init()
	{
		$this->getDatatype()->setLabel( $this->getFieldTranslation("_number",array(),"") );
	}
}

class AnwContentFieldSettings_enum extends AnwContentFieldSettings_datatype
{
	function initDatatype()
	{
		$oDatatype = new AnwDatatype_enum();
		$oDatatype->setDefaultValue("");
		return $oDatatype;
	}
	
	function setEnumValues($asEnumValues)
	{
		$this->getDatatype()->setEnumValues($asEnumValues);
	}	
	
	function setEnumValuesFromList($asEnumValues)
	{
		$this->getDatatype()->setEnumValuesFromList($asEnumValues);
	}
	
	//when field is multiple, remove redondant values
	function getValuesFromPost($sSuffix)
	{
		$asFieldValues = parent::getValuesFromPost($sSuffix);
		return array_unique($asFieldValues);
	}
}

class AnwContentFieldSettings_checkboxGroup extends AnwContentFieldSettings_enum implements AnwStructuredContentField_renderedAsMultiple
{
	function initDatatype()
	{
		$oDatatype = new AnwDatatype_checkboxGroup();
		$oDatatype->setDefaultValue("");
		return $oDatatype;
	}
}

//----------------------------------------------

abstract class AnwContentFieldSettings_container extends AnwContentFieldSettings implements AnwStructuredContentField_composed
{
	final protected function renderEditInput($asInputSubcontentAndRendered, $sSuffix, $nInputNumber)
	{
		$oSubContent = $asInputSubcontentAndRendered[AnwStructuredContent::IDX_SUBCONTENT];
		$sRendered = $asInputSubcontentAndRendered[AnwStructuredContent::IDX_RENDERED];		
		return $this->renderEditInputComposed($oSubContent, $sRendered, $sSuffix);
	}
	
	//overridable
	protected function renderEditInputComposed($oSubContent, $sRendered, $sSuffix)
	{
		$HTML = <<<EOF

<div class="contentfield_container">
	$sRendered
</div> <!-- end contentfield_container -->
EOF;
		return $HTML;
	}
	
	
	
	
	final protected function renderCollapsedInput($asInputSubcontentAndRendered, $sSuffix)
	{
		$oSubContent = $asInputSubcontentAndRendered[AnwStructuredContent::IDX_SUBCONTENT];
		$sRendered = $asInputSubcontentAndRendered[AnwStructuredContent::IDX_RENDERED];	
		return $this->renderCollapsedInputComposed($oSubContent, $sRendered, $sSuffix);
	}
	protected function renderCollapsedInputComposed($oSubContent, $sRendered, $sSuffix){
		return $sRendered;
	}
	
	function hasDefaultValue()
	{
		return true;
	}	
	function getDefaultValue()
	{
		$sDefaultValue = "";		
		$aoSubFields = $this->getContentFields();
		foreach ($aoSubFields as $oSubField)
		{
			$sFieldName = $oSubField->getName();
			$sDefaultValue .= '<'.$sFieldName.'>';			
			
			if ($oSubField->hasDefaultValues())
			{
				$amDefaultValues = $oSubField->getDefaultValues();
				foreach ($amDefaultValues as $mDefaultValue)
				{
					$sDefaultValue .= '<'.AnwStructuredContent::XMLNODE_VALUE.'>';
					$sDefaultValue .= $mDefaultValue;
					$sDefaultValue .= '</'.AnwStructuredContent::XMLNODE_VALUE.'>';
				}
			}
			$sDefaultValue .= '</'.$sFieldName.'>';
		}		
		return $sDefaultValue;
	}
	
	//used by action_edit
	function updateSuffix($sSuffix, $nInputNumber=-1)
	{
		$sSuffix .= '-'.$this->getName();
		if ($this->isMultiple())
		{
			if ($nInputNumber == -1) throw new AnwUnexpectedException("updateSuffix no inputNumber but multiple");
			$sSuffix .= $nInputNumber;
		}
		return $sSuffix;
	}
	
	function getValuesFromPost()
	{
		throw new AnwUnexpectedException("should never go here");
	}
	
	function getValueFromConfigItem($mCfgItem)
	{
		throw new AnwUnexpectedException("getValueFromConfigItem on container");
	}
	function getConfigItemForContent($oContent)
	{
		$cfg = array();
		foreach ($this->getSubContentFields() as $oSubContentField)
		{
			$sFieldName = $oSubContentField->getName();
			if ($oSubContentField->isMultiple())
			{
				$cfg[$sFieldName] = array();
				$aoSubContents = $oContent->getSubContents($sFieldName);
				foreach ($aoSubContents as $oSubContent)
				{
					$cfg[$sFieldName] = $oSubContentField->getConfigItemForValue($oSubContent);
				}
			}
			else
			{
				$oSubContent = $oContent->getSubContent($sFieldName);
				$cfg[$sFieldName] = $oSubContentField->getConfigItemForValue($oSubContent);
			}
		}
		return $cfg;
	}
	
	final function doTestContentFieldValue($mContentFieldValue)
	{
		$oSubContent = $this->rebuildContentFromXml($mContentFieldValue);
		$this->doTestContentFieldValueComposed($oSubContent);
	}
	//overridable
	protected function doTestContentFieldValueComposed($oSubContent){}
}



abstract class AnwContentFieldSettings_tab extends AnwContentFieldSettings_container implements AnwStructuredContentField_tab
{
	final protected function renderEditInputComposed($oSubContent, $sRendered, $sSuffix)
	{
		return $sRendered;
	}
	
	//a tab is never collapsed
	final function isCollapsed()
	{
		return false;
	}
	
	final function setMultiplicity($oMultiplicity)
	{
		throw new AnwUnexpectedException("no multiplicity can be set for tabs");
	}
	
	function renderEditTab($sSuffix)
	{
		$sTabDivId = 'tab_'.$this->getInputName($sSuffix);
		return $this->getComponent()->tpl()->renderEditTab($this->getEditTab(), $sTabDivId);
	}
	
	//overridable
	function getEditTab()
	{
		$HTML = $this->getFieldLabel();
		return $HTML;
	}
}




class AnwContentFieldSettings_mysqlconnexion extends AnwContentFieldSettings_container implements AnwISettings_mysqlconnexion
{
	function init()
	{
		parent::init();
		$oContentField = new AnwContentFieldSettings_string(self::FIELD_USER);
		$oContentField->setMandatory(true);
		$this->addContentField($oContentField);
		
		$oContentField = new AnwContentFieldSettings_password(self::FIELD_PASSWORD);
		$oContentField->setMandatory(true);
		$this->addContentField($oContentField);
		
		$oContentField = new AnwContentFieldSettings_string(self::FIELD_HOST);
		$this->addContentField($oContentField);
		
		$oContentField = new AnwContentFieldSettings_string(self::FIELD_DATABASE);
		$oContentField->setMandatory(true);
		$this->addContentField($oContentField);
		
		$oContentField = new AnwContentFieldSettings_string(self::FIELD_PREFIX);
		$this->addContentField($oContentField);
	}
	
	protected function doTestContentFieldValueComposed($oSubContent)
	{
		$sUser = $oSubContent->getContentFieldValue(self::FIELD_USER);
		$sPassword = $oSubContent->getContentFieldValue(self::FIELD_PASSWORD);
		$sHost = $oSubContent->getContentFieldValue(self::FIELD_HOST);
		$sDatabase = $oSubContent->getContentFieldValue(self::FIELD_DATABASE);
		$sPrefix = $oSubContent->getContentFieldValue(self::FIELD_PREFIX);
		
		try {
			$oDbLinkTest = AnwMysql::getInstance($sUser, $sPassword, $sHost, $sDatabase, $sPrefix);
		}
		catch(AnwDbConnectException $e) {
			$sError = AnwComponent::g_editcontent("err_contentfield_mysqlconnexion_dbconnect", array("details"=>$e->getMessage()));
			throw new AnwInvalidContentFieldValueException($sError);
		}
		//print "<br/>Test::user={$sUser}, pwd={$sPassword}, db={$sDatabase}, host={$sHost}, prefix={$sPrefix}<br/>";
		
	}
	
	function renderCollapsedInputComposed($oSubContent, $sRendered, $sSuffix)
	{
		$sUser = $oSubContent->getContentFieldValue(self::FIELD_USER);
		$sPassword = $oSubContent->getContentFieldValue(self::FIELD_PASSWORD);
		$sHost = $oSubContent->getContentFieldValue(self::FIELD_HOST);
		$sDatabase = $oSubContent->getContentFieldValue(self::FIELD_DATABASE);
		$sPrefix = $oSubContent->getContentFieldValue(self::FIELD_PREFIX);
		
		$asParameters = array('user'=>'<i>'.$sUser.'</i>', 'host'=>'<i>'.$sHost.'</i>', 'database'=>'<i>'.$sDatabase.'</i>', 'prefix'=>'<i>'.$sPrefix.'</i>');
		return $this->getFieldTranslation("_collapsed", $asParameters);
	}
}
?>